@@ -9,7 +9,7 @@ use Capture::Tiny qw(capture_merged);
 use Passwd::Keyring::OSXKeychain::PasswordTranslate qw(read_security_encoded_passwd);
 
 # TODO: considering we use Capture::Tiny, maybe drop IPC::System::Simple
-#       and move to Capture::Tiny altogether (note that this means 
+#       and move to Capture::Tiny altogether (note that this means
 #       checking exit status and raising exceptions). Or at least
 #       drop all capturex.
 
@@ -19,11 +19,11 @@ Passwd::Keyring::OSXKeychain - Password storage implementation based on OSX/Keyc
 
 =head1 VERSION
 
-Version 0.30
+Version 0.3001
 
 =cut
 
-our $VERSION = '0.30';
+our $VERSION = '0.3001';
 
 =head1 WARNING
 
@@ -66,10 +66,10 @@ C<Passwd::Keyring::Auto> package).
 
 =head2 new(app=>'app name', group=>'passwords folder')
 
-Initializes the processing. Croaks if osxkeychain keyring does not 
+Initializes the processing. Croaks if osxkeychain keyring does not
 seem to be available.
 
-Handled named parameters: 
+Handled named parameters:
 
 - app - symbolic application name (not used at the moment, but can be
   used in future as comment and in prompts, so set sensibly)
@@ -107,18 +107,28 @@ sub new {
           unless $self->{keychain} =~ /\.keychain$/;
     }
 
-    # Some test operation
-    my $reply = capturex($self->_make_sys_args(
-        "-q",
-        "show-keychain-info",
-        "default-keychain"));
+    # Making some security call to make sure it exists and works
+    # (we should die if Keychain is not available/not working)
+    my $reply = capturex(
+        $self->{security},
+        "list-keychains");
+    # list-keychains returns quoted, indented by 4 spaces list like:
+    #    "/Users/maros/Library/Keychains/login.keychain"
+    #    "/Library/Keychains/System.keychain"
+    # So far let's just test whether reply seems to contain anything.
+    unless($reply =~ /\.keychain/) {
+        croak("OSXKeychain not available: security program $self->{security} seems unaware of any keychains (security list-keychains returned '$reply')\n");
+    }
+
+    # Another idea is to test specific keychain
+    # -q show-keychain-info «name»
 
     return $self;
 }
 
 # Prepares args by prefixing with command and suffixing with keychain
 # if specified
-sub _make_sys_args {
+sub _make_keychainop_cmd {
     my ($self, @args) = @_;
     unshift @args, $self->{security};
     push @args, $self->{keychain} if $self->{keychain};
@@ -127,7 +137,7 @@ sub _make_sys_args {
 
 =head2 set_password(username, password, realm)
 
-Sets (stores) password identified by given realm for given user 
+Sets (stores) password identified by given realm for given user
 
 =cut
 
@@ -135,7 +145,7 @@ sub set_password {
     my ($self, $user_name, $user_password, $realm) = @_;
 
     # TODO: maybe use -l (label) instead of -D
-    systemx($self->_make_sys_args(
+    systemx($self->_make_keychainop_cmd(
         "-q", # quiet
         "add-generic-password",
         "-a", $user_name,
@@ -148,7 +158,7 @@ sub set_password {
         ));
 }
 
-# Parser for "-g" find-generic-password variant 
+# Parser for "-g" find-generic-password variant
 sub _parse_password_from_find_output {
     my ($text) = @_;
 
@@ -179,7 +189,7 @@ sub get_password {
     if($USING_ENCODED_OUTPUT) {
         my $reply = capturex(
             [0, 44],
-            $self->_make_sys_args(
+            $self->_make_keychainop_cmd(
                 "-q", # quiet
                 "find-generic-password",
                 "-a", $user_name,
@@ -187,13 +197,13 @@ sub get_password {
                 "-D", $self->{group}, # "kind", can be used to match so let be
                 "-w", # display (encoded) password only
                ));
-        return read_security_encoded_password($reply);
+        return read_security_encoded_passwd($reply);
     }
     else {
         my $reply = capture_merged  {
             runx(
                 [0, 44],   # Legal exit values. Some CpanTesters report 44 on password not found
-                $self->_make_sys_args(
+                $self->_make_keychainop_cmd(
                     "-q", # quiet
                     "find-generic-password",
                     "-a", $user_name,
@@ -210,14 +220,14 @@ sub get_password {
 
 Removes given password (if present)
 
-Returns how many passwords actually were removed 
+Returns how many passwords actually were removed
 
 =cut
 
 sub clear_password {
     my ($self, $user_name, $realm) = @_;
 
-    my $reply = systemx($self->_make_sys_args(
+    my $reply = systemx($self->_make_keychainop_cmd(
         "delete-generic-password",
         "-a", $user_name,
         "-s", $realm,
@@ -246,7 +256,7 @@ Marcin Kasperski
 
 =head1 BUGS
 
-Please report any bugs or feature requests to 
+Please report any bugs or feature requests to
 issue tracker at L<https://bitbucket.org/Mekk/perl-keyring-osxkeychain>.
 
 =head1 SUPPORT
@@ -277,4 +287,3 @@ See http://dev.perl.org/licenses/ for more information.
 
 
 1; # End of Passwd::Keyring::OSXKeychain
-
